package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.ShiroUtils;
import com.taauav.front.entity.UserDep;
import com.taauav.front.mapper.UserDepMapper;
import com.taauav.front.service.IUserAdminService;
import com.taauav.front.service.IUserDepService;
import com.taauav.front.vo.UserDepListVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.validation.Validator;
import java.util.*;

/**
 * <p>
 * 部门管理表 服务实现类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-25
 */
@Service
public class UserDepServiceImpl extends BaseServiceImpl<UserDepMapper, UserDep> implements IUserDepService {

    @Resource
    private Response response;
    @Autowired
    private UserDepMapper userDepMapper;
    @Resource
    private Validator validator;
    @Autowired
    private IUserAdminService adminService;

    /**
     * 获取部门数据
     *
     * @return
     */
    @Override
    public Response getList() {
        // 获取子级
        List<UserDepListVo> userDepListVoList = this.getChildCityList(0);
        return response.success(userDepListVoList);
    }

    /**
     * 获取子级部门列表
     *
     * @param pid 上级ID
     * @return
     */
    public List<UserDepListVo> getChildCityList(Integer pid) {
        QueryWrapper<UserDep> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", pid);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");
        List<UserDep> userDepList = userDepMapper.selectList(queryWrapper);
        List<UserDepListVo> userDepListVoList = new ArrayList<>();
        if (!userDepList.isEmpty()) {
            userDepList.forEach(item -> {
                UserDepListVo userDepListVo = new UserDepListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, userDepListVo);
                // 判断是否有子级
                List<UserDepListVo> childrenList = this.getChildCityList(item.getId());
                if (!childrenList.isEmpty()) {
                    userDepListVo.setChildrenList(childrenList);
                    userDepListVo.setHasChildren(true);
                } else {
                    userDepListVo.setHasChildren(false);
                }
                userDepListVoList.add(userDepListVo);
            });
        }
        return userDepListVoList;
    }

    /**
     * 添加部门
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response add(UserDep entity) {
        if (entity == null) {
            return response.failure("实体对象不能为空");
        }
        if (entity.getPid() == null) {
            entity.setLevel(1);
        } else {
            UserDep userDep = userDepMapper.selectById(entity.getPid());
            if (userDep != null) {
                entity.setLevel(userDep.getLevel() + 1);
            }
        }
        entity.setCreateTime(DateUtil.now());
        entity.setCreateUser(ShiroUtils.getAdminId());
        int result = userDepMapper.insert(entity);
        if (result == 0) {
            return response.failure("添加失败");
        }
        return response.success("添加成功");
    }

    /**
     * 获取机构详情
     *
     * @param id 机构ID
     * @return
     */
    @Override
    public Response info(Integer id) {
        if (id == null) {
            return response.failure("机构ID不存在");
        }
        UserDep entity = this.getById(id);
        if (entity == null) {
            return response.failure("机构信息不存在");
        }
        return response.success(entity);
    }

    /**
     * 编辑机构
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response edit(UserDep entity) {
        if (entity == null || entity.getId() == null) {
            return response.failure("实体对象不能为空");
        }
        // 同名不给添加
        QueryWrapper<UserDep> queryWrapper = new QueryWrapper<>();
        queryWrapper.ne("id", entity.getId());
        queryWrapper.eq("name", entity.getName());
        queryWrapper.eq("level", entity.getLevel());
        queryWrapper.eq("mark", 1);
        UserDep userDep = userDepMapper.selectOne(queryWrapper);
        if (userDep != null) {
            return response.failure("同级已存在该部门名称");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(DateUtil.now());
        boolean result = this.updateById(entity);
        if (!result) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 删除机构
     *
     * @param id 机构ID
     * @return
     */
    @Override
    public Response delete(Integer id) {
        if (id == null) {
            return response.failure("机构ID不存在");
        }
        UserDep entity = userDepMapper.selectById(id);
        if (entity == null) {
            return response.failure("机构信息不存在");
        }
        // 校验是否存在子级
        QueryWrapper<UserDep> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", entity.getId());
        queryWrapper.eq("mark", 1);
        Integer count = userDepMapper.selectCount(queryWrapper);
        if (count > 0) {
            return response.failure("存在子级，无法删除");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(DateUtil.now());
        entity.setMark(0);
        boolean result = this.updateById(entity);
        if (!result) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response setStatus(UserDep entity) {
        if (entity.getId() == null) {
            return response.failure("设备ID不存在");
        }
        if (entity.getStatus() == null || entity.getStatus() <= 0) {
            return response.failure("状态不存在");
        }
        int result = userDepMapper.updateById(entity);
        if (result == 0) {
            return response.failure("设置失败");
        }
        return response.success("设置成功");
    }
}
